//..............................................................................
// -*- Mode: Java; -*-
//                          Package   : DataBrowser
// DataView.java            Author    : Deanna Evans (dke),
//                                      Ed Kraemer (eck) and,
//                                      Mark Pumar (map)
//
//  This file is part of DataBrowser.
//
//  DataBrowser is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
//  USA.
//
//..............................................................................
//    THIS IS PART OF THE CONFIGURABLE INTERFACE AND HAS BEEN WRITTEN AS A
//    USGS SPECIFIC IMPLEMENTATION.
//..............................................................................
//
package DataView;

import javax.swing.*;
import java.awt.*;
import java.awt.image.*;
import java.io.*;
import java.text.*;
import java.lang.*;
import DataSet.*;


//.............................................................................
//
// Description:  DataView class
//
public class DataView extends DataSet
{
   //...........................................................................
   //
   // Description:  The DataView default constructor creates default data and
   //               image when the data browser application is initialized.
   //
   public DataView(
                    int Nx,  // in: image width
                    int Ny   // in: image height
                  )
   {
      // Call base class constructor; set additional class members
      super(Nx,Ny);
      mResX = mResY = 9;
      mNz   =  mResY*100/3;
      if( mDataMax < mSCALE) mScale = 0;
      else mScale = 1;
   }
   //...........................................................................
   //
   // Description:  The DataView constructor creates an image from the input
   //               byte data array obtained from the server.
   //
   public DataView(
                    byte[] data    // in: data from server
                  )
   {
      // Call base class constructor; set additional class members
      super(data);
      mNz =  mResY*100/3;
      if( mDataMax < mSCALE) mScale = 0;
      else mScale = 1;
   }
   //...........................................................................
   //
   // Description: The dataViewHist method is used to construct a height
   //              difference histogram of the data.  The histogram is used to
   //              compute the image intensity scale factor.
   //
   private void dataViewHist()
   {
      int i;        // counter
      int j;        // counter
      int dh;       // height difference

      // Create new histogram
      mHz = new int[2*mNz+1];
      for(i = 0; i <= 2*mNz; i++) mHz[i] = 0;

      // Compute height differences and store in histogram
      for(i = 1; i < mNx; i++)
      {
         for(j = 1; j < mNy; j++)
         {
            // Height difference in x-direction
            dh = (int) (dataSetValue(i, j+1)-dataSetValue(i, j-1));
            dh += mNz;
            if(dh < 0) dh = 0;
            if(dh > 2*mNz) dh = 2*mNz;
            mHz[dh] += 1;

            // Height difference in y-direction
            dh =  (int) (dataSetValue(i+1, j)-dataSetValue(i-1, j));
            dh += mNz;
            if(dh < 0) dh = 0;
            if(dh > 2*mNz) dh = 2*mNz;
            mHz[dh] += 1;
         }
      }
      // Sum histogram
      for(i = 1; i <= 2*mNz; i++)
      {
         mHz[i] = mHz[i] + mHz[i - 1];
      }
   }
   //...........................................................................
   //
   // Description:  The dataViewSmax method computes the image intensity
   //               scale factor from the height difference histogram.
   //
   private void dataViewSmax()
   {
      int    i    = 0;   // counter
      double sumH = 0;   // weighted height difference sum
      double sumI = 0;   // square of weights

      // Compute weighted histogram sum
      for(i = -mNz; i <= mNz; i++)
      {
         sumH += (i*mHz[i+mNz]);
         sumI += (i*i);
      }
      // Calculate intensity scale factor
      mSmax = sumH/(mHz[2*mNz]*sumI);
   }
   //...........................................................................
   //
   // Description: The dataViewContour returns the contour color index at the
   //              input grid point (i,j).
   //
   private int dataViewContour(
                                int i,  // in: x-coordinate of grid point
                                int j   // in: y-coordinate of grid point
                              )
   {
      int k;  // counter

      // Search countour heights for correct index
      for(k = 1; k <= mNumCon; k++)
      {
         if(mScale == 0) if( dataSetValue(i,j) <= mMeterLo[k]) return(k);
         else if(dataSetValue(i,j) <= mMeterHi[k]) return(k);
      }
      return(0);
   }
   //...........................................................................
   //
   // Description: The dataSetImageCreate method is re-implemented from the
   //              base class to compute a shaded relief image of the terrain.
   //
   public void dataSetImageCreate()
   {
      double        x,y,z;                // sun and slope factors
      int           Alpha, Beta;          // brightness coeffs
      int           bzero, f;             // brightness coeffs
      int           FAC;                  // dcale factor
      int           Dx, Dy;               // yerrain slopes
      int           i, j, k;              // vounters
      int           imgVal;               // image brightness value
      int           red;                  // red component of image
      int           green;                // green component of image
      int           blue;                 // nlue component of image
      int           gamma;                // image gamma
      int           counter = 0;          // array index

      mPixels = new int[mNx*mNy];         // create new array to hold image pixels

      if (mType == mDATA)                 // data sent by server is data, not image
      {
         // Calculate optimal scale factor
         dataViewHist();
         dataViewSmax();

         // Compute sun vector
         x = mSmax*Math.sin(Math.PI*mSUN/180.0);
         y = mSmax*Math.cos(Math.PI*mSUN/180.0);

         // Compute integer scale factor
         z = ( x < y ? x : y);
         FAC = 1;
         while( z < 1.0)
         {
            z   = 10.0*z;
            FAC = 10*FAC;
         }

         // Compute image intensity components
         bzero  = (FAC*mNBINS)/2;
         f      = FAC*mNBINS - bzero;
         Alpha  = (int) (f*x);
         Beta   = (int) (f*y);
         bzero += ((Alpha + Beta)/2);

         // Compute image from height data
         for (i = 1; i < mNx-1; i++)
         {
            for (j = 1; j < mNy-1; j++)
            {
               // Get contour index and height differences
               k  = dataViewContour(i, j);
               Dx = (int) (dataSetValue(i+1, j)-dataSetValue(i-1, j));
               Dy = (int) (dataSetValue(i, j+1)-dataSetValue(i, j-1));

               // Determine pixel intensity
               imgVal = (int) ((bzero + Alpha*Dx + Beta*Dy)/FAC);
               imgVal = (imgVal > 0 ? imgVal : 0);
               imgVal = (imgVal < mNBINS ? imgVal : mNBINS);
               imgVal = mNBINS - imgVal;

               // Compute color at this pixel
               red   = mRed[k]*imgVal;
               green = mGreen[k]*imgVal;
               blue  = mBlue[k]*imgVal;
               gamma = 255 - imgVal;

               // Store pixel intensity in image array
               mPixels[i + mNx*(mNy-j-1)] = (gamma << 24) + (red << 16) + (green << 8) + blue;
            }
         }

         // Fill edges. Use the brightness values of the next-to-edge rows
         // and columns.
         for (j = 0; j < mNy; j++)
         {
            mPixels[j*mNx] = mPixels[j*mNx+1];
            mPixels[j*mNx+mNx-1] = mPixels[j*mNx+mNx-2];
         }

         for (i = 0; i < mNx; i++)
         {
            mPixels[i] = mPixels[mNx+i];
            mPixels[i+(mNy-1)*mNx] = mPixels[mNx*(mNy-2)+i];
         }
      }
      else if (mType == mIMAGE && mIntData != null)  // if data is image, load into
      {                                              // pixel array
         for(i = 0; i < mNx; i++)
         {
            for(j = 0; j < mNy; j++)
            {
               mPixels[counter] = mIntData[i][j];
               counter++;
            }
         }
      }

      // Create image and display on screen
      mImage = createImage(new MemoryImageSource(mNx,mNy,mPixels,0,mNx));
      repaint();
   }
   //...........................................................................
   //
   // Description:  The dataSetToLocation converts a point from data grid
   //               coordinates to data coordinates.  This method is
   //               re-implemented from the base class.
   //
   public String dataSetToLocation(
                                    Point pt   // in: point
                                  )
   {
      Point  grid = pt;                 // grid point
      String loc;                       // location in lat-lon (output)
      String dirLon, dirLat;            // direction (East-West; North-South)
      int    degLon, minLon, secLon;    // degrees, minutes, seconds of longitude
      int    degLat, minLat, secLat;    // degrees, minutes, seconds of latitude

      // Invert y coordinate
      grid.y = mNy - 1 - grid.y;

      // Get data value
      grid.x = (grid.x >= 0 ? grid.x : 0);
      grid.x = (grid.x < mNx ? grid.x : (mNx-1));
      grid.y = (grid.y >= 0 ? grid.y : 0);
      grid.y = (grid.y < mNy ? grid.y : (mNy-1));

      // Convert to arc sec
      grid.x = 3600*mOrgX + mResX*grid.x;
      grid.y = 3600*mOrgY + mResY*grid.y;

      // Compute directions
      if(grid.x < 0)
      {
         grid.x = -grid.x;
         dirLon = "W";
      }
      else dirLon = "E";

      if(grid.y < 0)
      {
         grid.y = -grid.y;
         dirLat = "S";
      }
      else dirLat = "N";

      // Determine degrees
      degLon = (int) ((double) grid.x/3600.0);
      degLat = (int) ((double) grid.y/3600.0);

      grid.x -= 3600*degLon;
      grid.y -= 3600*degLat;

      // Compute minutes
      minLon = (int) ((double) grid.x/60.0);
      minLat = (int) ((double) grid.y/60.0);

      // Compute seconds
      secLon = grid.x - 60*minLon;
      secLat = grid.y - 60*minLat;

      // Convert location to string
      loc = degLat+" "+minLat+" "+secLat+" "+dirLat+"   "+degLon+" "+minLon+" "+secLon+" "+dirLon;

      // Return location
      return(loc);
   }
   //...........................................................................
   //
   // Description: The dataSetWrite method writes the data to an output file
   //              in a user chosen format.  This method is re-implemented
   //              from the base class.
   //
   public void dataSetWrite(
                              String fileName   // in: output file name
                           )
   {
      int i,j;           // counters
      short pad = 0;     // header pad

      try
      {
         // Open output file
         FileOutputStream fout = new FileOutputStream(fileName);
         DataOutputStream dout = new DataOutputStream(new BufferedOutputStream(fout));

         // Write header information
         dout.writeShort((short) (mNx-1));          // width
         dout.writeShort((short) (mNy-1));          // height
         dout.writeShort((short) (mResY*100/3));    // resolution
         dout.writeShort((short) (mOrgX));          // longitude origin
         dout.writeShort((short) (mOrgY));          // latitude origin
         dout.writeShort(pad);                      // padding
         dout.writeShort(pad);                      // padding
         dout.writeShort((short) (mResX));          // longitude resolution
         dout.writeShort((short) (mResY));          // latitude resolution

         for(i = 9; i < 128; i++) dout.writeShort(pad);  // remaining header padding

         // write data
         for(i = 0; i < mNx; i++)
         {
            for(j = 0; j < mNy; j++)
            {
               dout.writeShort((short) dataSetValue(i,j));
            }
         }
         // close output file
         dout.close();
      }
      catch(IOException e){ }
   }
   //...........................................................................
   // Data Members
   //
   private        int [] mHz           = null;         // height difference histogram
   private        int    mNz           = 0;            // length of mHz
   private        double mSmax         = 0;            // intenisty scale factor
   private        int    mScale        = 0;            // contour scale
   private static int    mSCALE        = 4500;         // height scale threshold
   private static int    mBFAC         = 8;            // number of colors/contours
   private static int    mNBINS        = 28;           // number of color bins in image
   private static double mSUN          = 45.0;         // sun angle in degrees
   private static int [] mMeterLo      = {0,0,200, 500,1000,1500,2000,3000,6000};
   private static int [] mMeterHi      = {0,0,500,1000,2000,3000,4000,6000,9000};
   public  static String mPORT         = "2809";       // default port
   public  static String mHOST         = "127.0.0.1";  // local host
   public  static String mOBJECT_ID    = "Data";       // object ID
   public  static String mOBJECT_KIND  = "Object";     // object kind
   public  static String mCONTEXT_ID   = "DataReader"; // context ID
   public  static String mCONTEXT_KIND = "Context";    // context kind
}
//..............................................................................
//  * END * END * END * END * END * END * END * END * END * END * END * END *
//..............................................................................
