//..............................................................................
// -*- Mode: C++; -*-
//                          Package   : DataHandler
// DataHandler.cpp          Author    : Deanna Evans (dke),
//                                      Ed Kraemer (eck) and,
//                                      Mark Pumar (map) 
//
//  This file is part of DataHandler.
//
//  DataHandler is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
//  USA.
//
//..............................................................................
// Description: The DataHandler functions are used to read, render, and supply 
//              data to the server. This class constitutes the configurable 
//              portion of the server.  Its public methods are called by the
//              JAVA wrapper and server to pass data to the browser.
//              This implementation reads a terrain elevation data file
//              in the proprietary FTD format.
//
//    THIS IS PART OF THE CONFIGURABLE INTERFACE AND HAS BEEN WRITTEN AS A
//    USGS SPECIFIC IMPLEMENTATION.
//..............................................................................
//
#include "DataHandler.h"

// Global variables
const double BASE_GRID = 100.0;  // grid base
const int    LENGTH    = 256;    // header length in bytes

struct dataHeader
{
    int mLength,   // header length
        mType,     // data type, isData or isImage
        mKind,     // data format, isInt or isReal 
        mSize,     // unit data size in byte
        mNx,       // data width
        mNy,       // data height
        mLat,      // data x-origin location
        mLon,      // data y-origin location
        mLatRes,   // data x-resolution
        mLonRes,   // data y-resolution
        mLatExt,   // data x-extension
        mLonExt,   // data y-extension
        mHmin,     // data minimum value
        mHmax,     // data maximum value
        mNz,       // spacing factor 
        mFileSize; // size of input data file
};
// Global variables
static unsigned char    *dataPtr = 0;  // pointer to data

// Function prototypes
static int               IsNum(char  c);
static struct dataHeader ReadA(FILE *mFile);
static void              ReadB(FILE *mFile);
static void              dataIntToByte(unsigned char *head,int value);
static void              loadHeader(int res,struct dataHeader header);

extern "C"
{
   //...........................................................................
   //
   // Description:  Read file as Image (NOT Implemented)
   //
   DLL_EXPORT unsigned char * readFileAsImage(
                                              const char *fileName, // in: filename 
                                              int res,              // in: requested resolution                
                                              int &length           // out: data length
                                             )
   {
      // Set the data length to zero
      length = 0;
      // Reset the pointer to the data, if not null
      if(dataPtr != 0) delete [] dataPtr;
      dataPtr = 0;
      return(dataPtr);
   }

   //...........................................................................
   //
   // Description:  Read file as data
   //
   DLL_EXPORT unsigned char * readFileAsData(
                                             const char *fileName, // in: filename
                                             int res,              // in: requested resolution
                                             int &length           // out: data length   
                                            )
   {
      // Set the data length to zero
      length = 0;
      // Return if fileName or res are invalid
      if((fileName == 0) || (res <= 0)) return(0);

      // Open the file
      FILE *fin = fopen(fileName,"r");
      if(fin == 0) return(0);

      // Read the header
      struct dataHeader header = ReadA(fin); // USGS header

      // Local variables
      int    i,j,k;                          // Counters                 
      int    counter = 0;                    // array location counter
      int    Nx = (header.mNx-1)/res + 1;    // width of data
      int    Ny = (header.mNy-1)/res + 1;    // height of data
      char   val[2];                         // character array
      short  el;                             // elevation 

      // Reset the data pointer, if not null
      if(dataPtr != 0) delete [] dataPtr;
      dataPtr = new unsigned char[2*Nx*Ny+4*header.mLength];
      if(dataPtr == 0) return(0);
  
      // Read in the heights by columns
      for(i = 0; i < header.mNx; i++)    
      { 
         // Read the USGS data
         ReadB(fin);

         for(j = 0; j < header.mNy; j++)
         {
            fscanf(fin,"%hd",&el);
            // Read every ith and jth member based on the requested resolution
            if(((j % res) == 0) && ((i % res) == 0))
            { 
               memcpy(val, &el, 2); 
#ifdef __x86__
               // Byte flip the data
               for(k = 0; k <=1; k++) dataPtr[counter+k+4*header.mLength] = val[1-k];
#else
               // Do not byte flip the data
               for(k = 0; k <=1; k++) dataPtr[counter+k+4*header.mLength] = val[k];
#endif
               // Record the maximum and minimum data value
               if(el < header.mHmin) header.mHmin = el;
               if(el > header.mHmax) header.mHmax = el;
               counter += 2;
            }
         }
      }

      // Adjust the sizes to reflect the requested resolution
      header.mNx = Nx;
      header.mNy = Ny;
      header.mLonRes = header.mLonRes*res;
      header.mLatRes = header.mLatRes*res;
      header.mFileSize = 2*Nx*Ny+4*header.mLength;

      // Set the length of the data to the file size
      length = header.mFileSize;

      // Fill the header 
      loadHeader(res,header);
      fclose(fin);
      return(dataPtr);
   }

   //...........................................................................
   //
   // Description:  Read the first part of the header for the USGS data
   //
   DLL_EXPORT int getFileInfo(
                              const char *fileName, // in: filename
                              int &width,           // out: width of data in file 
                              int &height,          // out: height of data in file
                              int &length           // out: header length
                             )
   {
      // Initialize the width and height
      width  = 0;
      height = 0;
      length = 0;

      // Return if fileName is invalid
      if(fileName == 0) return(0);

      // Open the file
      FILE *fin = fopen(fileName,"r");
      if(fin == 0) return(0);

      // Read the header
      struct dataHeader header = ReadA(fin);

      fclose(fin);

      // Set the return variables
      width  = header.mNx;
      height = header.mNy;
      length = sizeof(int)*header.mLength;

      return(header.mFileSize);
   }
} // extern C

//..............................................................................
//
// Description:  Get header information
//
void loadHeader(
                int res,                 // in: requested resolution
                struct dataHeader header // in: data header struct
               )
{
   // Return if invalid data pointer
   if(dataPtr == 0) return;

   // Set the header of the byte stream
   dataIntToByte(dataPtr,    header.mLength);
   dataIntToByte(dataPtr+ 4, header.mType);
   dataIntToByte(dataPtr+ 8, header.mKind);
   dataIntToByte(dataPtr+12, header.mSize);
   dataIntToByte(dataPtr+16, header.mNx);
   dataIntToByte(dataPtr+20, header.mNy);
   dataIntToByte(dataPtr+24, header.mLon);
   dataIntToByte(dataPtr+28, header.mLat);
   dataIntToByte(dataPtr+32, header.mLonRes);
   dataIntToByte(dataPtr+36, header.mLatRes);
   dataIntToByte(dataPtr+40, header.mLonExt);
   dataIntToByte(dataPtr+44, header.mLatExt);
   dataIntToByte(dataPtr+48, header.mHmin);
   dataIntToByte(dataPtr+52, header.mHmax);
   dataIntToByte(dataPtr+56, header.mNz);
   dataIntToByte(dataPtr+60, header.mFileSize);
}

//..............................................................................
//
// Description:  Convert int to byte
//
void dataIntToByte(
                   unsigned char *head, // out: character pointer
                   int value            // in: integer value
                  )
{
   // Return if invalid character pointer
   if(head == 0) return;

   // Local variables
   char val[4];  // character array
   int  i;       // loop counter

   memcpy(val, &value,   4);     
#ifdef __x86__
   // Byte flip the data
   for(i = 0; i <=3; i++) head[i]    = val[3-i]; 
#else
   // Do not byte flip the data
   for(i = 0; i <=3; i++) head[i]    = val[i]; 
#endif
}

//..............................................................................
//
// Description:  Read the first part of the header for the USGS data
//
struct dataHeader ReadA(
                        FILE *mFile  // in: file pointer
                       )
{
   char   buf[256];             // buffer for reading
   char   res[256];             // buffer for reading
   char   quad_field_name[144]; // buffer for name in file
   int    i,j,k;                // counters
   int    end,beg;              // flags
   short  level_code,           // level code of file
          pattern_code,         // pattern code of file
          plan_ref_sys_code,    // reference system of file
          zone_code,            // zone code of file
          units_code1,          // planimetric units code of file
          units_code2,          // elevation units code of file
          num_sides,            // number of polygon sides of file
          accuracy_code,        // accuracy code of file
          num_row,              // number of rows in file
          num_col;              // number of columns in file
   double map_proj_pram[15],    // map parameters of file
          corners[4][2],        // corners of file 
          angle,                // reference angle of file
          min_el,               // minimum elevation in file
          max_el,               // maximum elevation in file
          resolution[3],        // resolution array of file
          grid,                 // grid 
          gridFac;              // grid factor
   struct dataHeader header;    // header structure
   
   // Zero out the return structure
   memset(&header,0,sizeof(struct dataHeader));

   // Return if file pointer is bad
   if(mFile == 0) return(header);

   // Read the quadrangle name field 
   for(i = 0; i < 144; i++) fscanf(mFile,"%c",&quad_field_name[i]);
  
   // Read the level code, pattern code, planimetric reference system code, zone code 
   fscanf(mFile,"%hd",&level_code); 
   fscanf(mFile,"%hd",&pattern_code); 
   fscanf(mFile,"%hd",&plan_ref_sys_code); 
   fscanf(mFile,"%hd",&zone_code);
   
   // Read the map projection parameters  
   for(j = 0; j < 15; j++)
   {
      // Clear the buffer 
      for(i = 0; i < 256; i++) buf[i] = 0;
      beg = end = 0;
      // Loop while the character is not a "number"
      while(!beg) 
      {
         fscanf(mFile,"%c",&buf[0]);
         beg = IsNum(buf[0]);
      }
      i = 1;
      // Loop while the character is a "number" and buffer is not exceeded
      while(!end && (i < 256))
      {
         fscanf(mFile,"%c",&buf[i]);
         end = !IsNum(buf[i]);
         if((buf[i] == 'd') || (buf[i] == 'D')) buf[i] = 'E';
         ++i;
      }
      // Set the map projection parameters
      sscanf(buf,"%le",&map_proj_pram[j]);     
   }
       
   // Read the ground planimetric and elevation units codes 
   fscanf(mFile,"%hd",&units_code1);
   fscanf(mFile,"%hd",&units_code2);
  
   // Read the number of polygon sides defining coverage of DEM 
   fscanf(mFile,"%hd",&num_sides);
  
   // Read the corners of polygon 
   for(j = 0; j < 4; j++)
   {
      for(k = 0; k < 2; k++)
      {
         // Clear the buffer 
         for(i = 0; i < 256; i++) buf[i] = 0;
         beg = end = 0;
         // Loop while the character is not a "number"
         while(!beg) 
         {
            fscanf(mFile,"%c",&buf[0]);
            beg = IsNum(buf[0]);
         }
         i = 1;
         // Loop while the character is a "number" and buffer is not exceeded
         while(!end && ( i < 256))
         {
            fscanf(mFile,"%c",&buf[i]);
            end = !IsNum(buf[i]);
            if((buf[i] == 'd') || (buf[i] == 'D')) buf[i] = 'E';
            ++i;
         }
         // Set the corners of the polygon
         sscanf(buf,"%le",&corners[j][k]);
      }
   }
     
   // Read the min and max elevation and reference angle 
   for(j = 0; j < 3; j++) 
   {
      // Clear the buffer
      for(i = 0; i < 256; i++) buf[i] = 0;
      beg = end = 0;
      // Loop while the character is not a "number"
      while(!beg) 
      {
         fscanf(mFile,"%c",&buf[0]);
         beg = IsNum(buf[0]);
      }
      i = 1;
      // Loop while the character is a "number" and buffer is not exceeded
      while(!end && (i < 256))
      {
         fscanf(mFile,"%c",&buf[i]);
         end = !IsNum(buf[i]);
         if((buf[i] == 'd') || (buf[i] == 'D')) buf[i] = 'E';
         ++i;
      }
      // Set the correct variable depending on the loop counter
      if(j == 2) sscanf(buf,"%le",&angle);
      if(j == 1) sscanf(buf,"%le",&max_el);
      if(j == 0) sscanf(buf,"%le",&min_el);
    }

    // Read the accuracy code, spatial resolution  
    // Clear the buffers
    for(i = 0; i < 256; i++) buf[i] = 0;
    for(i = 0; i < 256; i++) res[i] = 0;
    beg = end = 0;

    // Loop while the character is not a "number"
    while(!beg) 
    {
      fscanf(mFile,"%c",&buf[0]);
      beg = IsNum(buf[0]);
    }
    i = 1;
    // Loop while the character is a "number" and buffer is not exceeded
    while(!end && ( i < 256))
    {
      fscanf(mFile,"%c",&buf[i]);
      end = !IsNum(buf[i]);
      if((buf[i] == 'd') || (buf[i] == 'D')) buf[i] = 'E';
      ++i;
    }
      
    // Add spaces between resolutions 
    j = 0;
    for(i = 0; i < 256; i++)
    {
      res[j] = buf[i];
      if(buf[i+2] == '.')
      {
         ++j;
         if(j > 255) j = 255;
         res[j] = ' ';
      }
      ++j;
      if(j > 255) j = 255;
    }
    
    // Set the resolutions    
    sscanf(res,"%hd %le %le %le",&accuracy_code,&resolution[0],
                                 &resolution[1],&resolution[2]);
   
    // Read the number of rows and columns    
    fscanf(mFile,"%hd",&num_row);
    fscanf(mFile,"%hd",&num_col);

    // Store the information collected into the header
    header.mNx      = num_col;
    header.mNy      = 1201;
    header.mNz      = 100*(units_code1/3);
    grid    = 100*(units_code1/3);
    header.mLon     = (int) (corners[0][0]/3600);
    header.mLat     = (int) (corners[0][1]/3600);
    header.mLonRes  = (int) resolution[0];
    header.mLatRes  = (int) resolution[1];
    gridFac = grid/BASE_GRID;
    header.mLatExt  = (int) ((3*(header.mNy-1)*gridFac)/3600);
    header.mLonExt  = (int) ((3*(header.mNx-1)*gridFac)/3600);

    header.mHmin   =  (int) min_el;      //  Min height
    header.mHmax   =  (int) max_el;      //  Max height
    header.mLength = LENGTH/sizeof(int); // header length
    header.mType   = isData;
    header.mKind   = isInt;
    header.mSize   = sizeof(short);
    header.mFileSize = LENGTH + (header.mNx*header.mNy)*sizeof(short); // Size of the file

    return(header);
}

//..............................................................................
//
// Description:  Read the second part of the header for the USGS data
//
void ReadB(
           FILE *mFile // in: file pointer
          )
{
   char    buf[256];       // buffer for reading
   int     i,j;            // counter
   int     beg,end;        // flags
   short   row_no,         // row number currently reading in file
           col_no,         // column number currenlty reading in file
           num_rows,       // number of rows in file
           num_cols;       // number of columns in file
   double  lon,            // current longitude in file
           lat,            // current latitude in file
           el_local_datum, // elevation datum in file
           el_min,         // current minimum elevation in file
           el_max;         // current maximum elevation in file

   // Return if file pointer is bad
   if(mFile == 0) return;
  
   // Scan the file for the current row and column and 
   // the total number of rows and columns
   fscanf(mFile,"%hd",&row_no); 
   fscanf(mFile,"%hd",&col_no); 
   fscanf(mFile,"%hd",&num_rows); 
   fscanf(mFile,"%hd",&num_cols);

   // Read the min and max elevation 
   for(j = 0; j < 5; j++) 
   {
      // Clear the buffer
      for(i = 0; i < 256; i++) buf[i] = 0;
      beg = end = 0;
      // Loop while the character is not a "number"
      while(!beg) 
      {
         fscanf(mFile,"%c",&buf[0]);
         beg = IsNum(buf[0]);
      }
      i = 1;
      // Loop while the character is a "number" and the buffer is not exceeded
      while(!end)
      {
         fscanf(mFile,"%c",&buf[i]);
         end = !IsNum(buf[i]);
         if((buf[i] == 'd') || (buf[i] == 'D')) buf[i] = 'E';
         ++i;
      }
      // Set the correct variable depending on the loop counter
      if(j == 0) sscanf(buf,"%le",&lon);
      if(j == 1) sscanf(buf,"%le",&lat);
      if(j == 2) sscanf(buf,"%le",&el_local_datum);
      if(j == 3) sscanf(buf,"%le",&el_min);
      if(j == 4) sscanf(buf,"%le",&el_max);
   }    
}

//..............................................................................
//
// Description:  Determine if this is a number for the USGS data
//
int IsNum(
          char  c  // in: character
         )
{
   if(c == '0') return(1);
   if(c == '1') return(1);
   if(c == '2') return(1);
   if(c == '3') return(1);
   if(c == '4') return(1);
   if(c == '5') return(1);
   if(c == '6') return(1);
   if(c == '7') return(1);
   if(c == '8') return(1);
   if(c == '9') return(1);
   if(c == '-') return(1);
   if(c == '+') return(1);
   if(c == '.') return(1);
   if(c == 'd') return(1);
   if(c == 'D') return(1);
   if(c == 'e') return(1);
   if(c == 'E') return(1);

   return(0);
}
//..............................................................................
//*END * END * END * END * END * END * END * END * END * END * END * END * END *
//..............................................................................
