//..............................................................................
// -*- Mode: C++; -*-
//                          Package   : DataServer
// DataInterface.cpp        Author    : Deanna Evans (dke),
//                                      Ed Kraemer (eck) and,
//                                      Mark Pumar (map)
//
//  This file is part of DataServer.
//
//  DataServer is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
//  USA.
//
//..............................................................................
// Description: The DataInterface class provides access to functions for reading
//              the data contained in the shared library or dll.
//..............................................................................
//
// Include files
#include "DataInterface.h"
#include <iostream.h>

// Function prototypes
typedef unsigned char * (* ReadFileAsPtr)     (const char *fileName, int res, int &length);
typedef int             (* GetFileInfoPtr)    (const char *fileName,int &width, int &height, int &length);
typedef unsigned long   (* GetMsgSizePtr)     (void);
typedef std::string     (* GetConfigInfoPtr)  (void);

//..............................................................................
//
// Description:  Class constructor
//
DataInterface::DataInterface(void)
{
#ifdef __WIN32__
   // Load the DLL if this is a windows platform
   mLib = LoadLibrary("DataHandler.dll");
#endif
}

//..............................................................................
//
// Description:  Class destructor
//
DataInterface::~DataInterface(void)
{
#ifdef __WIN32__
   // Free the library if this is a windows platform
   if(mLib != NULL) FreeLibrary(mLib);
#endif
}

//..............................................................................
//
// Description:  Data retrieval method
//
unsigned char * DataInterface::diGetData(
                                         const char *fileName, // in: file name 
                                         int res,              // in: requested resolution  
                                         const char *type,     // in: data type (Image or Data) 
                                         int &length           // out: data length
                                        )
{
   // Initialize the length
   length = 0;

   // Return if type is invalid, else call the appropriate function
   if(type == 0) return(0);
   else if(strcmp(type,"Data") == 0)
      return(diReadFileAsData(fileName, res, length));
   else if(strcmp(type,"Image") == 0)
      return(diReadFileAsImage(fileName, res, length));
   else return(0);
}

//..............................................................................
//
// Description:  Data retrieval method for data type
//
unsigned char * DataInterface::diReadFileAsData(
                                                const char *fileName, // in: file name
                                                int res,              // in: requested resolution
                                                int &length           // out: data length 
                                               )
{
#ifdef __WIN32__
   // Return if library is invalid
   if(mLib == NULL) return(0);

   // Get the function pointer
   ReadFileAsPtr ReadFileAsData = (ReadFileAsPtr) GetProcAddress(mLib,"readFileAsData");

   // Return if function pointer is invalid   
   if(ReadFileAsData == NULL) return(0);

   // Get data
   return((ReadFileAsData)(fileName,res,length));
#else
   // Get data
   return(readFileAsData(fileName,res,length));
#endif
}

//..............................................................................
//
// Description:  Data retrieval method for image type
//
unsigned char * DataInterface::diReadFileAsImage(
                                                 const char *fileName, // in: file name
                                                 int res,              // in: requested resolution
                                                 int &length           // out: data length 
                                                )
{
#ifdef __WIN32__
   // Return if library is invalid
   if(mLib == NULL) return(0);

   // Get the function pointer
   ReadFileAsPtr ReadFileAsImage = (ReadFileAsPtr) GetProcAddress(mLib,"readFileAsImage");

   // Return if function pointer is invalid
   if(ReadFileAsImage == NULL) return(0);

   // Get image 
   return((ReadFileAsImage)(fileName,res,length));
#else
   // Get image 
   return(readFileAsImage(fileName,res,length));
#endif
}

//..............................................................................
//
// Description:  Method to get the width, height and file size of file
//
int DataInterface::diGetFileInfo(
                                 const char *fileName, // in: file name
                                 int &width,           // out: width of file 
                                 int &height,          // out: height of file
                                 int &length           // out: header length
                                )
{
#ifdef __WIN32__
   // Return if library is invalid
   if(mLib == NULL) return(0);

   // Get the function pointer
   GetFileInfoPtr GetFileInfo = (GetFileInfoPtr) GetProcAddress(mLib,"getFileInfo");

   // Return if function pointer is invalid   
   if(GetFileInfo == NULL) return(0);

   // Get the width, height, and file size
   return((GetFileInfo)(fileName,width,height,length));
#else
   // Get the width, height, and file size
   return(getFileInfo(fileName,width,height,length));
#endif
}

//..............................................................................
//
// Description:  Get the maximum message size
//
unsigned long   DataInterface::diGetMsgSize(void)
{
#ifdef __WIN32__
   // Return if library is invalid
   if(mLib == NULL) return(0);

   // Get the function pointer
   GetMsgSizePtr GetMsgSize = (GetMsgSizePtr) GetProcAddress(mLib,"getMsgSize");
   
   // Return if function pointer is invalid   
   if(GetMsgSize == NULL) return(0);

   // Get the maximum message size   
   return((GetMsgSize)());
#else
   // Get the maximum message size   
   return(getMsgSize());
#endif
}

//..............................................................................
//
// Description:  Get the CORBA context kind
//
std::string DataInterface::diGetContextKind(void)
{
#ifdef __WIN32__
   // Return if library is invalid
   if(mLib == NULL) return(std::string(""));

   // Get the function pointer   
   GetConfigInfoPtr GetContextKind = (GetConfigInfoPtr) GetProcAddress(mLib,"getContextKind");

   // Return if function pointer is invalid   
   if(GetContextKind == NULL) return(std::string(""));

   // Get the context kind
   return((GetContextKind)());
#else
   // Get the context kind
   return(getContextKind());
#endif
}

//..............................................................................
//
// Description:  Get the CORBA context ID
//
std::string DataInterface::diGetContextID(void)
{
#ifdef __WIN32__
   // Return if library is invalid
   if(mLib == NULL) return(std::string(""));

   // Get the function pointer
   GetConfigInfoPtr GetContextID = (GetConfigInfoPtr) GetProcAddress(mLib,"getContextID");

   // Return if function pointer is invalid   
   if(GetContextID == NULL) return(std::string(""));

   // Get the context ID   
   return((GetContextID)());
#else
   // Get the context ID   
   return(getContextID());
#endif
}

//..............................................................................
//
// Description:  Get the CORBA object kind
//
std::string DataInterface::diGetObjectKind(void)
{
#ifdef __WIN32__
   // Return if library is invalid
   if(mLib == NULL) return(std::string(""));

   // Get the function pointer
   GetConfigInfoPtr GetObjectKind = (GetConfigInfoPtr) GetProcAddress(mLib,"getObjectKind");

   // Return if function pointer is invalid   
   if(GetObjectKind == NULL) return(std::string(""));

   // Get the object kind   
   return((GetObjectKind)());
#else
   // Get the object kind   
   return(getObjectKind());
#endif
}

//..............................................................................
//
// Description:  Get the CORBA object ID
//
std::string DataInterface::diGetObjectID(void)
{
#ifdef __WIN32__
   // Return if library is invalid
   if(mLib == NULL) return(std::string(""));

   // Get the function pointer    
   GetConfigInfoPtr GetObjectID = (GetConfigInfoPtr) GetProcAddress(mLib,"getObjectID");

   // Return if function pointer is invalid   
   if(GetObjectID == NULL) return(std::string(""));

   // Get the object ID
   return((GetObjectID)());
#else
   // Get the object ID
   return(getObjectID());
#endif
}

//..............................................................................
//
// Description:  Get the file extension for the data file
//
std::string DataInterface::diGetFileExt(void)
{
#ifdef __WIN32__
   // Return if library is invalid
   if(mLib == NULL) return(std::string(""));

   // Get the function pointer   
   GetConfigInfoPtr GetFileExt = (GetConfigInfoPtr) GetProcAddress(mLib,"getFileExt");

   // Return if function pointer is invalid   
   if(GetFileExt == NULL) return(std::string(""));

   // Get the file extension
   return((GetFileExt)());
#else
   // Get the file extension
   return(getFileExt());
#endif
}

//..............................................................................
//
// Description:  Get the path delimiter for the server
//
std::string DataInterface::diGetPathDelimiter(void)
{
#ifdef __WIN32__
   // Return if library is invalid
   if(mLib == NULL) return(std::string(""));

   // Get the function pointer   
   GetConfigInfoPtr GetPathDelimiter = (GetConfigInfoPtr) GetProcAddress(mLib,"getPathDelimiter");

   // Return if function pointer is invalid   
   if(GetPathDelimiter == NULL) return(std::string(""));

   // Get the path delimiter   
   return((GetPathDelimiter)());
#else
   // Get the path delimiter   
   return(getPathDelimiter());
#endif
}
//..............................................................................
//*END * END * END * END * END * END * END * END * END * END * END * END * END *
//..............................................................................
