//..............................................................................
// -*- Mode: C++; -*-
//                          Package   : DataServer
// DataReader.cpp           Author    : Deanna Evans (dke),
//                                      Ed Kraemer (eck) and,
//                                      Mark Pumar (map)
//  
//  This file is part of DataServer.
//
//  DataServer is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
//  USA.
//
//..............................................................................
// Description: The DataReader class contains the functions called by the
//              JAVA client.
//..............................................................................
//
// Include files
#include "DataReader.h"
#include "FileService.h"
#include "Data.h"
#include <iostream.h>

// System dependent include files
#ifdef __WIN32__
#include <strstrea.h>
#else
#include <strstream.h>
#endif


using _CORBA_MODULE Data;

//..............................................................................
//
// Description:  Method to return the path delimiter
//
CORBA::Long DataReader::getPathDelimiter(
                                          CORBA::String_out pathDel // out: path delimiter
                                        )
{
   // Check if the interface pointer is valid
   if(mDataInterfacePtr != 0)
   {
      // The pointer is valid, get the path delimiter
      mPathDel = mDataInterfacePtr->diGetPathDelimiter();
      pathDel  = mPathDel.c_str();
      return(1);
   }
   else return(0);
   
}

//..............................................................................
//
// Description:  Function to get the list of files 
//
CORBA::Long DataReader::getFiles(
                                  const char* dirName,       // in: directory name
                                  CORBA::String_out fileList // out: file list
                                 )
{
   // Check if the interface pointer is valid
   if(mDataInterfacePtr == 0) return(0);

   // The pointer is valid, get the list of files
   mTheFiles = FileService::fsGetFileList(std::string(dirName),
                                          mDataInterfacePtr->diGetFileExt());
   fileList = mTheFiles.c_str();
   return(1);
}

//..............................................................................
//
// Description:  Method to get the current directory location 
//
CORBA::Long DataReader::getLocation(
                                     CORBA::String_out dirLoc // out: directory
                                   )
{
   // Call the function to get the current directory
   mTheDir = FileService::fsGetAbsDir();
   dirLoc = mTheDir.c_str();
   return(1);
}

//..............................................................................
//
// Description:  Method to get the file width, height and size 
//
CORBA::Long DataReader::getFileResolution( const char* file,    // in: data file name
                                           CORBA::Long& width,  // out: data width
                                           CORBA::Long& height, // out: data height
                                           CORBA::Long& length  // out: header length
                                          )
{
   // Check for valid interface pointer and filename
   if((mDataInterfacePtr == 0) || (file == 0)) return(0);

   // Initialize the width and height
   int w = 0, h = 0, l = 0;

   // Call the function to get the file width, height, and size
   CORBA::Long size = mDataInterfacePtr->diGetFileInfo(file, w, h,l);
   width  = w;
   height = h;
   length = l;
   return(size);
}

//..............................................................................
//
// Description:  Method to get the input data 
//
CORBA::Long DataReader::getData( const char* file,      // in: data file name
                                 CORBA::Long res,       // in: data resolution
                                 const char* type,      // in: data or image
                                 UnboundedData_out vals // out: data
                               )
{
   // Check for valid interface pointer, filename, and type
   if((mDataInterfacePtr == 0) || (file == 0) || (type == 0)) return(0);

   // Initialize the length of the file
   int length = 0;

   // Call the function to get the data
   unsigned char *dataPtr = mDataInterfacePtr->diGetData(file, res, type, length);
   
   // Check for valid data pointer
   if(dataPtr == 0) return(0);

   // Assign data to output sequence
   vals = new UnboundedData(length,length,dataPtr,0);

   return(1);
}

//..............................................................................
//
// Description:  Class constructor 
//
DataReader::DataReader(void)
{
   mDataInterfacePtr = new DataInterface;
}

//..............................................................................
//
// Description:  Class destructor 
//
DataReader::~DataReader(void)
{
   if(mDataInterfacePtr != 0) delete mDataInterfacePtr;
}

//..............................................................................
//
// Description:  Method to get the maximum message size and set it for the ORB 
//
void DataReader::drSetServerMessageSize(void)
{
   // Check for valid interface pointer
   if(mDataInterfacePtr == 0) return;

   // Call the function to get the maximum message size
   unsigned long msgSize = mDataInterfacePtr->diGetMsgSize();

   // If valid message size, set the maximum message size for the ORB
   if(msgSize != 0) omniORB::MaxMessageSize(msgSize);
}

//..............................................................................
//
// Description:  Method to get the context kind 
//
std::string DataReader::drGetContextKind(void)
{
   // Check for valid interface pointer
   if(mDataInterfacePtr == 0) return(std::string(""));
   else return(mDataInterfacePtr->diGetContextKind());
}

//..............................................................................
//
// Description:  Method to get the context ID 
//
std::string DataReader::drGetContextID(void)
{
   // Check for valid interface pointer
   if(mDataInterfacePtr == 0) return(std::string(""));
   else return(mDataInterfacePtr->diGetContextID());
}

//..............................................................................
//
// Description:  Method to get the object kind 
//
std::string DataReader::drGetObjectKind(void)
{
   // Check for valid interface pointer
   if(mDataInterfacePtr == 0) return(std::string(""));
   else return(mDataInterfacePtr->diGetObjectKind());
}

//..............................................................................
//
// Description:  Method to get the object ID 
//
std::string DataReader::drGetObjectID(void)
{
   // Check for valid interface pointer
   if(mDataInterfacePtr == 0) return(std::string(""));
   else return(mDataInterfacePtr->diGetObjectID());
}
//..............................................................................
//*END * END * END * END * END * END * END * END * END * END * END * END * END *
//..............................................................................
