//..............................................................................
// -*- Mode: C++; -*-
//                          Package   : DataServer
// DataServer.cpp           Author    : omniORB
//
//  This file is part of DataServer.
//
//  DataServer is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
//  USA.
//
//..............................................................................
// Description: This is the object implementation.
//
// Usage: DataServer
//
//        On startup, the object reference is registered with the
//        COS naming service. The client uses the naming service to
//        locate this object.
//
//        The name which the object is bound to is as follows:
//              root  [context]
//               |
//              test  [context] kind [my_context]
//               |
//              Data  [object]  kind [Object]
//
// Acknowledgements: This code was obtained from "The omniORB version 3.0 User's
//                   Guide", May 2000, pages 31 - 34.
//
//..............................................................................
//
// Include files
#include "DataReader.h"
#include <iostream.h>

// Function prototypes
static CORBA::Boolean bindObjectToName(CORBA::ORB_ptr, CORBA::Object_ptr, DataReader* dataPtr);

//..............................................................................
//
// Description:  Main method for the DataServer
//
int main(int argc, char **argv)
{
   try 
   {
      // Initialize ORB and obtain a reference to an ORB object
      CORBA::ORB_var orb = CORBA::ORB_init(argc, argv, "omniORB3");

      // Obtain a reference to a base (root) CORBA object and
      // activate it.  It is now available to all clients.
      CORBA::Object_var obj = orb->resolve_initial_references("RootPOA");
      PortableServer::POA_var poa = PortableServer::POA::_narrow(obj);

      // Create an instance of the DataReader and activate it
      DataReader* mydata = new DataReader();
      PortableServer::ObjectId_var mydataid = poa->activate_object(mydata);
      
      // Obtain a reference to the object, and register it in
      // the naming service.
      obj = mydata->_this();
      if( !bindObjectToName(orb, obj, mydata) ) return(1);
      mydata->_remove_ref();

      // Increase allowable message size   
      mydata->drSetServerMessageSize();

      // Active server; its initially in a holding state, make it active
      PortableServer::POAManager_var pman = poa->the_POAManager();
      pman->activate();

      // Wait for and process requests
      cout << "Server running..." << endl;
      orb->run();
      orb->destroy();
   }
   
   // Catch exceptions
   catch(CORBA::SystemException &e) 
   {
      cerr << "Caught CORBA::SystemException."  << endl;
   }
   catch(CORBA::Exception &e) 
   {
      cerr << "Caught CORBA::Exception." << endl;
   }
   catch(omniORB::fatalException& fe) 
   {
      cerr << "Caught omniORB::fatalException:" << endl;
      cerr << "  file: " << fe.file() << endl;
      cerr << "  line: " << fe.line() << endl;
      cerr << "  mesg: " << fe.errmsg() << endl;
   }
   catch(...) 
   {
      cerr << "Caught unknown exception." << endl;
   }

   return(0);
}

//..............................................................................
//
// Description:  Method to bind the object to the name
//
static CORBA::Boolean bindObjectToName(CORBA::ORB_ptr orb, CORBA::Object_ptr objref, DataReader* dataPtr)
{
   CosNaming::NamingContext_var rootContext;

   try 
   {
      // Obtain a reference to the root context of the Name service:
      CORBA::Object_var obj;
      obj = orb->resolve_initial_references("NameService");

      // Narrow the reference returned.
      rootContext = CosNaming::NamingContext::_narrow(obj);
      if( CORBA::is_nil(rootContext) ) 
      {
         cerr << "Failed to narrow the root naming context." << endl;
         return(0);
      }
   }
   catch(CORBA::ORB::InvalidName& ex) 
   {
      // This should not happen!
      cerr << "Service required is invalid [does not exist]." << endl;
      return(0);
   }

   try 
   {
      // Bind a context called "test" to the root context:
      CosNaming::Name contextName;
      contextName.length(1);
      contextName[0].id   = dataPtr->drGetContextID().c_str();    // string copied
      contextName[0].kind = dataPtr->drGetContextKind().c_str();  // string copied
      // Note on kind: The kind field is used to indicate the type
      // of the object. This is to avoid conventions such as that used
      // by files (name.type -- e.g. test.ps = postscript etc.)

      CosNaming::NamingContext_var testContext;
      try 
      {
         // Bind the context to root.
         testContext = rootContext->bind_new_context(contextName);
      }
      catch(CosNaming::NamingContext::AlreadyBound& ex) 
      {
         // If the context already exists, this exception will be raised.
         // In this case, just resolve the name and assign testContext
         // to the object returned:
         CORBA::Object_var obj;
         obj = rootContext->resolve(contextName);
         testContext = CosNaming::NamingContext::_narrow(obj);
         if( CORBA::is_nil(testContext) ) 
	  {
            cerr << "Failed to narrow naming context." << endl;
            return(0);
	  }
      }

      // Bind objref with name Data to the Context:
      CosNaming::Name objectName;
      objectName.length(1);
      objectName[0].id   = dataPtr->drGetObjectID().c_str();   // string copied
      objectName[0].kind = dataPtr->drGetObjectKind().c_str(); // string copied

      try 
      {
         testContext->bind(objectName, objref);
      }
      catch(CosNaming::NamingContext::AlreadyBound& ex) 
      {
         testContext->rebind(objectName, objref);
      }
      // Note: Using rebind() will overwrite any Object previously bound
      //       to /test/Echo with obj.
      //       Alternatively, bind() can be used, which will raise a
      //       CosNaming::NamingContext::AlreadyBound exception if the name
      //       supplied is already bound to an object.

      // Amendment: When using OrbixNames, it is necessary to first try bind
      // and then rebind, as rebind on it's own will throw a NotFoundexception if
      // the Name has not already been bound. [This is incorrect behaviour -
      // it should just bind].
   }
   // Catch exceptions
   catch(CORBA::COMM_FAILURE& ex) 
   {
      cerr << "Caught system exception COMM_FAILURE -- unable to contact the "
           << "naming service." << endl;
      return(0);
   }
   catch(CORBA::SystemException&) 
   {
      cerr << "Caught a CORBA::SystemException while using the naming service." << endl;
      return(0);
   }

   return(1);
}	
//..............................................................................
//*END * END * END * END * END * END * END * END * END * END * END * END * END *
//..............................................................................

