//..............................................................................
// -*- Mode: C++; -*-
//                          Package   : DataServer
// FileFind_unix.cpp        Author    : Deanna Evans (dke),
//                                      Ed Kraemer (eck) and,
//                                      Mark Pumar (map) 
//
//  This file is part of DataServer.
//
//  DataServer is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
//  USA.
//
//..............................................................................
// Description: The functions define herein are used to return the path and 
//              list of  files in a directory. 
//              Adapted from the Microsoft Foundation Classes C++ library.
//..............................................................................
//
#ifndef __WIN32__
// Include files
#include "FileFind.h"
#include <iostream.h>
#include <sys/stat.h>
#include <dirent.h>
#include <stdio.h>
#include <unistd.h>

// Defines
#define FALSE 0
#define TRUE  1

//..............................................................................
//
// Description:  Class constructor
//
FileFind::FileFind(
                   std::string dirName  //in: directory name
                  )
{
   // Initialize variables
   mDirName     = dirName;
   m_pFoundInfo = NULL;

   // Open the input directory
   m_hContext   = opendir(dirName.c_str());
}

//..............................................................................
//
// Description:  Class destructor
//
FileFind::~FileFind(void)
{
   // Close the current directory that was opened
   ffClose();
}

//..............................................................................
//
// Description:  Method to close an open directory
//
void FileFind::ffClose()
{
   //  Close the directory if the context is not null
   if( m_hContext != NULL) closedir((DIR *) m_hContext);

   // Reset variables
   m_hContext   = NULL;
   m_pFoundInfo = NULL;
}

//..............................................................................
//
// Description:  Method to find if a file with a particular extension exists
//               in the current open directory.
//
int FileFind::ffFindFile(
                         std::string fileExt  // in: file extension
                        )
{
   // If the context is null, open the directory the class was initialized with
   if(m_hContext   == NULL) m_hContext   = opendir(mDirName.c_str());

   // Check for valid context
   if(m_hContext   == NULL) return(FALSE);

   // Context was valid, read the directory
   m_pFoundInfo    = readdir((DIR *) m_hContext);

   // Check for valid file pointer
   if(m_pFoundInfo == NULL) return(FALSE);
   else return(TRUE);
}

//..............................................................................
//
// Description:  Method to find the next file
//
int FileFind::ffFindTheNextFile(void)
{
   // Check for valid pointers to the context and file info
   if((m_hContext  == NULL) || (m_pFoundInfo == NULL))
      return FALSE;
  
   // Pointers are valid, read the directory
   m_pFoundInfo    = readdir((DIR *) m_hContext);

   // Check for valid file pointer
   if(m_pFoundInfo == NULL) return(FALSE);
   else return(TRUE);
}

//..............................................................................
//
// Description:  Method to determine if the file has the same extension as input
//
int FileFind::ffHasExtension(
                             std::string fileExt  // in: file extension
                            ) const
{
   // Check if input string is "all files" 
   if(strcmp(fileExt.c_str(),"*.*") == 0) return(TRUE);     // anything OK

   // Check if current file pointed to is a directory
   if((ffIsDirectory() == TRUE) || (ffIsDots() == TRUE)) 
      return(FALSE);                                        // not a file

   // The current file pointed to is a file, check to see if has extension
   std::string fileName = ffGetFileName();                  // get file name
   if(strstr(fileName.c_str(),fileExt.c_str()+1) != NULL)   // check for extension
		return(TRUE);
  
   return(FALSE);
}

//..............................................................................
//
// Description:  Method to determine if this is a dots directory
//
int FileFind::ffIsDots(void) const
{
   // return TRUE if the file name is "." or ".." and
   // the file is a directory
   if((m_pFoundInfo != NULL) && ffIsDirectory())
   {
      if((strcmp(((struct dirent *)m_pFoundInfo)->d_name,".")   == 0) ||  // don't include ./ directory
         (strcmp(((struct dirent *)m_pFoundInfo)->d_name,"..")  == 0)) return(TRUE);  // don't include ../ directory     
   }
   return(FALSE);
}

//..............................................................................
//
// Description:  Method to determine if the file is a directory
//
int FileFind::ffIsDirectory(void) const
{
   // Check for valid file pointer
   if (m_pFoundInfo != NULL)
   {
      struct stat st;                                        // file information structure
      std::string fileName = mDirName;                       // build full path name to
      fileName +="/";                                        // to directory 
      fileName += ((struct dirent *)m_pFoundInfo)->d_name;   // information can be found

      if((lstat(fileName.c_str(),&st) >=  0) &&              // get file information
        ((st.st_mode & S_IFMT) == S_IFDIR)) return(TRUE);    // file is a directory
   }
   return(FALSE);
}

//..............................................................................
//
// Description:  Method to get the current file name 
//
std::string FileFind::ffGetFileName(void) const
{
   // Return a filename if the file info pointer is valid
   if(m_pFoundInfo != NULL)
      return(std::string(((struct dirent *)m_pFoundInfo)->d_name));      
   else return(std::string(""));
}
#endif
//..............................................................................
//*END * END * END * END * END * END * END * END * END * END * END * END * END *
//..............................................................................

