//..............................................................................
// -*- Mode: C++; -*-
//                          Package   : DataServer
// FileFind_windows.cpp     Author    : Deanna Evans (dke),
//                                      Ed Kraemer (eck) and,
//                                      Mark Pumar (map) 
//
//  This file is part of DataServer.
//
//  DataServer is free software; you can redistribute it and/or modify
//  it under the terms of the GNU General Public License as published by
//  the Free Software Foundation; either version 2 of the License, or
//  (at your option) any later version.
//
//  This program is distributed in the hope that it will be useful,
//  but WITHOUT ANY WARRANTY; without even the implied warranty of
//  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
//  GNU General Public License for more details.
//
//  You should have received a copy of the GNU General Public License
//  along with this program; if not, write to the Free Software
//  Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307,
//  USA.
//
//..............................................................................
// Description: The functions define herein are used to return the path and 
//              list of  files in a directory. 
//              Adapted from the Microsoft Foundation Classes C++ library.
//..............................................................................
//
// Include files
#include "FileFind.h"
#ifdef __WIN32__
#include <stdlib.h>
#include <string.h>
#include <windows.h>
#include <direct.h>

//..............................................................................
//
// Description:  Class constructor
//
FileFind::FileFind(
                   std::string dirName  //in: directory name
                  )
{
   // Initialize variables
   mDirName     = dirName;
   m_pFoundInfo = new WIN32_FIND_DATA;
   m_hContext   = NULL;
   mStartDir    = getcwd(0, 256);

   // Change directories to input directory, if not default
   if(strcmp(dirName.c_str(),".") != 0)
   {
      chdir(dirName.c_str());
   }
}

//..............................................................................
//
// Description:  Class destructor
//
FileFind::~FileFind(void)
{
   // Reset the file info pointer if not null
   if (m_pFoundInfo != NULL)
   {
      delete m_pFoundInfo;
      m_pFoundInfo = NULL;
   }

   // Close the current directory that was opened   
   ffClose();

   // Return to the initial starting directory
   if(mStartDir != 0)
   {
     chdir(mStartDir);
     free(mStartDir);
   }
}

//..............................................................................
//
// Description:  Method to close an open directory
//
void FileFind::ffClose()
{
   //  Close the directory if the context is not null
   if((m_hContext != NULL) && (m_hContext != INVALID_HANDLE_VALUE))
   {
      FindClose(m_hContext);
      m_hContext = NULL;
   }
}

//..............................................................................
//
// Description:  Method to find if a file with a particular extension exists
//               in the current open directory.
//
int FileFind::ffFindFile(
                         std::string fileExt  // in: file extension
                        )
{
   // Check for valid context
   if(m_pFoundInfo == NULL) return(FALSE);

   lstrcpy(((WIN32_FIND_DATA*) m_pFoundInfo)->cFileName, ext.c_str());

   // Find the first file with the input extension
   m_hContext = FindFirstFile(ext.c_str(), (WIN32_FIND_DATA*) m_pFoundInfo);

   // Check for valid context
   if(m_hContext == INVALID_HANDLE_VALUE)
   {
      ffClose();
      return(FALSE);
   }

   return(TRUE);
}

//..............................................................................
//
// Description:  Method to find the next file
//
int FileFind::ffFindTheNextFile(void)
{
   // Check for valid context and file info pointer
   if((m_hContext == NULL) || (m_pFoundInfo == NULL))
      return(FALSE);

   // Valid pointers, return next file found
   return FindNextFile(m_hContext, (LPWIN32_FIND_DATA) m_pFoundInfo);
}

//..............................................................................
//
// Description:  Method to determine if the file has the same extension as input
//
int FileFind::ffHasExtension(
                             std::string fileExt  // in: file extension
                            ) const
{
   // Check if input string is "all files" 
   if(strcmp(fileExt.c_str(),"*.*") == 0) return(TRUE);     // anything OK

   // Check if current file pointed to is a directory   
   if((ffIsDirectory() == TRUE) || (ffIsDots() == TRUE)) 
      return(FALSE);                                        // not a file
    
   // The current file pointed to is a file, check to see if has extension
   std::string fileName = ffGetFileName();                  // get file name
   if(strstr(fileName.c_str(),fileExt.c_str()+1) != NULL)   // check for extension
      return(TRUE);
  
   return(FALSE);
}

//..............................................................................
//
// Description:  Function to determine if this is a dots directory
//
int FileFind::ffIsDots(void) const
{
   // return TRUE if the file name is "." or ".." and
   // the file is a directory
   if((m_pFoundInfo != NULL) && ffIsDirectory())
   {
      LPWIN32_FIND_DATA pFindData = (LPWIN32_FIND_DATA) m_pFoundInfo;
      if (pFindData->cFileName[0] == '.')
      {
         if (pFindData->cFileName[1] == '\0' ||
            (pFindData->cFileName[1] == '.' &&
             pFindData->cFileName[2] == '\0'))
	  {
            return(TRUE);
	  }
      }
   }

   return(FALSE);
}

//..............................................................................
//
// Description:  Function to determine if the file is a directory
//
int FileFind::ffIsDirectory(void) const
{
   // Check for valid file pointer
   if (m_pFoundInfo != NULL)
   {
      if(((LPWIN32_FIND_DATA) m_pFoundInfo)->dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY)
         return(TRUE);
   }
   return(FALSE);
}

//..............................................................................
//
// Description:  Method to get the current file name 
//
std::string FileFind::ffGetFileName(void) const
{
   std::string ret("");

   // Return a filename if the file info pointer is valid
   if(m_pFoundInfo != NULL)
      return(std::string(((LPWIN32_FIND_DATA) m_pFoundInfo)->cFileName));
   else return(std::string(""));
}
#endif
//..............................................................................
//*END * END * END * END * END * END * END * END * END * END * END * END * END *
//..............................................................................

